# -*- encoding: utf-8 -*-
"""
MomentX
"""

__author__ = "Ulf Bartel <elastic.code@gmail.com>"
__version__ = "0.2.3"
__copyright__ = "Copyright (c) Ulf Bartel / Zach Williams"
__license__ = "New-style BSD"
__url__ = "https://github.com/berlincode/momentx"

import os
from datetime import datetime

import pytz
from .date import add, replace
from .core import parse_date, get_timezone

# you may use the tzlocal module to set local timezone
DEFAULT_TIMEZONE = os.environ.get('TZ', 'UTC')
DEFAULT_DATETIME_FORMAT_TZ = '%Y-%m-%d %H:%M:%S%z'

class MomentX(object):
    """A class to abstract date difficulties.
    """

    def __init__(self, *args, **kwargs):
        self._zero = kwargs.pop("zero", False)

        formula = None
        if len(args) == 1:
            date = args[0]
        elif len(args) == 2:
            date = args[0]
            formula = args[1]
        else:
            date = args

        if not date:
            # if no date is supplied take "now"
            date = datetime.utcnow().replace(tzinfo=pytz.utc)

        date = parse_date(date, formula, timezone=kwargs.get('timezone', None))

        self._date = date

    @classmethod
    def date(cls, *args, **kwargs):
        """Create a moment using the default timezone as default."""
        return cls(*args, timezone=kwargs.get("timezone", DEFAULT_TIMEZONE))

    @classmethod
    def now(cls, **kwargs):
        """Create a date from the present time."""
        return cls(timezone=kwargs.get("timezone", DEFAULT_TIMEZONE))

    @classmethod
    def utc(cls, *args, **kwargs): # pylint:disable=no-self-use
        """Create a date using the UTC time zone as default."""
        return cls(*args, timezone=kwargs.get("timezone", "UTC"))

    @classmethod
    def utcnow(cls): # pylint:disable=no-self-use
        """UTC equivalent to `now` function."""
        return cls(timezone="UTC")

    def add(self, key=None, amount=None, **kwds):
        """Add time to moment."""
        date = add(self._date, key=key, amount=amount, **kwds)
        return MomentX(date)

    def sub(self, key=None, amount=None, **kwds):
        """Just in case."""
        return self.subtract(key, amount, **kwds)

    def subtract(self, key=None, amount=None, **kwds):
        """Subtract time from the original moment."""
        return self.add(key=key, amount=amount, factor=-1, **kwds)

    def replace(self, **kwds):
        """A Pythonic way to replace various date attributes."""
        date = replace(self._date, **kwds)
        return MomentX(date)

    def zero(self):
        """Get rid of hour, minute, second, and microsecond information."""
        date = replace(self._date, hours=0, minutes=0, seconds=0, microseconds=0)
        return MomentX(date, zero=True)

    def astimezone(self, timezone="UTC"):
        """ change timezone """
        date = self._date.astimezone(get_timezone(timezone))
        return MomentX(date)

    def asutc(self):
        """ change timezone """
        return self.astimezone()

    def isoformat(self):
        """Return the date's ISO 8601 string."""
        return self._date.isoformat()

    def strftime(self, formula):
        """Display the moment in a given format."""
        return self._date.strftime(formula)

    def __repr__(self):
        if self._date is not None:
            return "<Moment(%s / %s)>" % (str(self), str(self._date.tzinfo))
        return "<Moment>"

    def __str__(self):
        # we want our default datetime representation without the timezone
        return self._date.strftime(DEFAULT_DATETIME_FORMAT_TZ)

    @property
    def year(self):
        return self._date.year

    @property
    def month(self):
        return self._date.month

    @property
    def day(self):
        return self._date.day

    @property
    def weekday(self):
        return self._date.isoweekday()

    @property
    def hour(self):
        return self._date.hour

    @property
    def hours(self):
        return self._date.hour

    @property
    def minute(self):
        return self._date.minute

    @property
    def minutes(self):
        return self._date.minute

    @property
    def second(self):
        return self._date.second

    @property
    def seconds(self):
        return self._date.second

    @property
    def microsecond(self):
        return self._date.microsecond

    @property
    def microseconds(self):
        return self._date.microsecond

    @property
    def tzinfo(self):
        return str(self._date.tzinfo)

    # pylint:disable=protected-access
    def __lt__(self, other):
        if isinstance(other, datetime):
            return self._date < other
        elif isinstance(other, type(self)):
            return self._date < other._date

    def __le__(self, other):
        if isinstance(other, datetime):
            return self._date <= other
        elif isinstance(other, type(self)):
            return self._date <= other._date

    def __eq__(self, other):
        if isinstance(other, datetime):
            return self._date == other
        elif isinstance(other, type(self)):
            return self._date == other._date

    def __ne__(self, other):
        if isinstance(other, datetime):
            return self._date != other
        elif isinstance(other, type(self)):
            return self._date != other._date

    def __gt__(self, other):
        if isinstance(other, datetime):
            return self._date > other
        elif isinstance(other, type(self)):
            return self._date > other._date

    def __ge__(self, other):
        if isinstance(other, datetime):
            return self._date >= other
        elif isinstance(other, type(self)):
            return self._date >= other._date
    # pylint:enable=protected-access



# -*- encoding: utf-8 -*-

from momentx import MomentX

# create a class if you need to support multiple default timezones or restricy
class MomentXWrapper(object):
    """
    Simple wrapper for momentx.

    1) wrapped the moment api to set a custom default_timezone
    2) additionally you may set a fixed 'now' which makes now()/utcnow() a
    constant during the lifetime of MomentXWrapper. This might also be convenient
    for testing
    """

    def __init__(self, default_timezone="UTC", now=None):
        self.default_timezone = default_timezone
        if now:
            self.args_now = [now]
        else:
            self.args_now = []

    def date(self, *args, **kwargs):
        """Create a moment using the default timezone as default."""
        return MomentX(*args, timezone=kwargs.get("timezone", self.default_timezone))

    def now(self, **kwargs):
        """Create a date from the present time."""
        return MomentX(*self.args_now, timezone=kwargs.get("timezone", self.default_timezone))

    def utc(self, *args, **kwargs): # pylint:disable=no-self-use
        """Create a date using the UTC time zone as default."""
        return MomentX(*args, timezone=kwargs.get("timezone", "UTC"))

    def utcnow(self): # pylint:disable=no-self-use
        """UTC equivalent to `now` function."""
        return MomentX(*self.args_now, timezone="UTC")



# -*- encoding: utf-8 -*-

import re
from datetime import datetime

import pytz
from six import string_types

from .mystrptime import mystrptime

def get_timezone(timezone):
    if timezone is None:
        timezone = pytz.utc # default
    elif isinstance(timezone, string_types):
        timezone = pytz.timezone(timezone)
    return timezone

def guess_formula(date):
    if re.match(r'.*:[0-9]{2}[+-]', date):
        # seems to have a timezone offset
        formula = '%Y-%m-%d %H:%M:%S%z'

    elif re.match(r'.*:[0-9\.]+[+-]', date):
        # seems to have milliseconds/microseconds and timezone offset
        formula = '%Y-%m-%d %H:%M:%S.%f%z'

    # support for complete dates
    elif ":" in date:
        formula = '%Y-%m-%d %H:%M:%S'
    else:
        formula = "%Y-%m-%d"

    return formula

def parse_date(date, formula=None, timezone=None, is_dst=None):

    if isinstance(date, string_types):
        # support for complete dates with timezone
        if not formula:
            formula = guess_formula(date)

        # the following special handling is only neccessary, because
        # we need %z for timezone offeset
        # TODO with python3.2 this won't be neccessary !!!

        date = mystrptime(date, formula)

    # this is like the default handling
    elif isinstance(date, list) or isinstance(date, tuple):
        if len(date) == 1:
            # Python datetime needs the month and day, too.
            date = [date[0], 1, 1]
        date = datetime(*date)
    elif isinstance(date, datetime):
        # this is already a datetime
        if date.tzinfo and (timezone is None):
            # if it has a timezone info we take this as default instead of utc
            timezone = date.tzinfo
    else:
        # epoch seconds
        date = datetime.utcfromtimestamp(date).replace(tzinfo=pytz.utc)

    timezone = get_timezone(timezone)

    if formula and formula.endswith('%z'):
        # a date with timezone is parsed as UTC - so we can convert it to local timezone
        return date.replace(tzinfo=pytz.utc).astimezone(timezone)
    elif date.tzinfo:
        # already has a timezone (e.g. if a datetime with tz is supplied)
        return date.astimezone(timezone)

    # the date was in local timezone
    return timezone.localize(date, is_dst=is_dst)



# -*- encoding: utf-8 -*-
import calendar
from datetime import timedelta

import pytz
from six import iteritems

def add_month(date, number):
    """Add a number of months to a date."""
    month = date.month - 1 + number
    return update_month(date, month)

def update_month(date, month):
    """Create a new date with a modified number of months."""
    year = date.year + month // 12
    month = month % 12 + 1
    day = min(date.day, calendar.monthrange(year, month)[1])
    return date.replace(year=year, month=month, day=day)

def add(date, key=None, amount=None, factor=1, use_localtime=None, force=False, **kwds):
    """Add time to the original moment."""
    if not key and not amount and len(kwds):
        items = iteritems(kwds)
    else:
        items = [(key, amount)]

    # TODO error (if not force) if time of day != 00:00:00 and using years, months, days
    # TODO use localtime for hours, minutes, seconds, milliseconds as default
    # add table to documentations

    # remember timezone
    tzinfo = date.tzinfo

    # normalize (just to be sure)
    date = tzinfo.normalize(date)

    # now we are working on utc (and localize back again later)
    date = date.replace(tzinfo=pytz.utc)

    for key, amount in items:
        if key == 'years' or key == 'year':
            date = add_month(date, amount * 12 * factor)
        elif key == 'months' or key == 'month':
            date = add_month(date, amount * factor)
        elif key == 'weeks' or key == 'week':
            date += timedelta(weeks=amount) * factor
        elif key == 'days' or key == 'day':
            date += timedelta(days=amount) * factor
        elif key == 'hours' or key == 'hour':
            date += timedelta(hours=amount) * factor
        elif key == 'minutes' or key == 'minute':
            date += timedelta(minutes=amount) * factor
        elif key == 'seconds' or key == 'second':
            date += timedelta(seconds=amount) * factor
        elif key == 'milliseconds' or key == 'millisecond':
            date += timedelta(milliseconds=amount) * factor
        elif key == 'microseconds' or key == 'microsecond':
            date += timedelta(microseconds=amount) * factor

    date = tzinfo.localize(date.replace(tzinfo=None), is_dst=None) # TODO is_dst
    return date

def replace(date, **kwds):
    """A Pythonic way to replace various date attributes."""
    for key, value in iteritems(kwds):
        if key == 'years' or key == 'year':
            date = date.replace(year=value)
        elif key == 'months' or key == 'month':
            date = date.replace(month=value)
        elif key == 'days' or key == 'day':
            date = date.replace(day=value)
        elif key == 'hours' or key == 'hour':
            date = date.replace(hour=value)
        elif key == 'minutes' or key == 'minute':
            date = date.replace(minute=value)
        elif key == 'seconds' or key == 'second':
            date = date.replace(second=value)
        elif key == 'microseconds' or key == 'microsecond':
            date = date.replace(microsecond=value)
        elif key == 'weekday':
            weekday = date.isoweekday()
            if value < 0:
                days = abs(weekday - value)
            else:
                days = weekday - value
            delta = date - timedelta(days)
            date = delta
    return date



# -*- encoding: utf-8 -*-
"""
 strptime with with timezone offset (%z) support for python2.7

 support timezone offset (%z) with strptime
 from python 3.2 this is supported natively
 pylint:disable=super-init-not-called

"""

import datetime
import pytz

# pylint:disable=super-init-not-called
class OffsetTime(pytz.tzinfo.StaticTzInfo):
    def __init__(self, offset):
        """A dumb timezone based on offset such as +0530, -0600, etc.
        """
        hours = int(offset[:3])
        minutes = int(offset[0] + offset[3:])
        self._utcoffset = datetime.timedelta(hours=hours, minutes=minutes)
# pylint:enable=super-init-not-called

def mystrptime(value, fmt='%Y-%m-%d %H:%M:%S%z'):
    if fmt.endswith('%z'):
        fmt = fmt[:-2]
        offset = value[-5:]
        value = value[:-5]
        return OffsetTime(offset).localize(datetime.datetime.strptime(value, fmt)).astimezone(pytz.utc).replace(tzinfo=None)

    return datetime.datetime.strptime(value, fmt)
